/*
*********************************************************************************
*           BasicAgent.h : Robocup 3D Soccer Simulation Team Zigorat            *
*                                                                               *
*  Date: 03/20/2007                                                             *
*  Author: Mahdi Hamdarsi                                                       *
*  Comments: This file contains class decleration BasicAgent which contains     *
*            several routines needed for thinkin' module                        *
*                                                                               *
*********************************************************************************
*/

/*! \file BasicAgent.h
<pre>
<b>File:</b>          BasicAgent.h
<b>Project:</b>       Robocup Soccer Simulation Team: Zigorat
<b>Authors:</b>       Mahdi Hamdarsi
<b>Created:</b>       03/20/2007
<b>Last Revision:</b> $ID$
<b>Contents:</b>      This file contains class decleration BasicAgent which contains
               several routines needed for thinkin' module
<hr size=2>
<h2><b>Changes</b></h2>
<b>Date</b>             <b>Author</b>          <b>Comment</b>
03/20/2007       Mahdi           Initial version created
</pre>
*/


#ifndef BASICAGENT_H
#define BASICAGENT_H
#define ShankSizeZ    0.11
#define  ThighSizeZ    0.14
#include "WorldModel.h"
#include "ActHandler.h"
#include "AgentSettings.h"
#include "Connection.h"
#include "Planner.h"
#include "string.h"
#include "Joint.h"
#define Max_Lenght 0.033
#define Max_Weight 0.01
#define Max_Hight 0.007
//#define Max_Lenght 0.04
//#define Max_Weight 0.005
//#define Max_Hight 0.009
/*! This class defines the skills that can be used by an agent. No
    functionality is available that chooses when to execute which skill, this
    is done in the Agent class. The WorldModel is used to determine the way
    in which the skills are performed. */
class BasicAgent
{
  protected:

	Joint				neck[2];
		Joint				arm[4][2];
		Joint				leg[6][2];
	VecPosition heap,ankleLeft,ankleRight;
	int walkState,walkPriod,StandState,shootState,kickState,TurnState;
	double walkGain ;
	double DISTANCE ;
	double ERROR_MOVE ;
	AngDeg ERROR_ANGLE ;
    bool                 m_Finished;        /*!< If this variable is true, agent will quit    */
    bool				 m_IsFirst;			//for Gyr		//abc 15
    RoboCupConnection  * connection;        /*!< Reference to Connection instance             */
    ActHandler         * ACT;               /*!< Reference to the ActHandler instance         */
    WorldModel         * WM;                /*!< Reference to WorldModle class                */
    AgentSettings      * AS;                /*!< Reference to AgentSettings class             */
    Logger             * Log;               /*!< Reference to use logger class                */
    bool              fallen;
    // Initialization
    bool  initAgentScene                    (                                                 );
    void  sendPlayerInformation             (                                                 );

    // Finalization
    void  sayGoodbye                        (                                                 );

    // Basic Skills
    void  moveToStrategicPosition           ( AngDeg              angBody = 0.00              );
    void  beam                              ( double              dx,
                                              double              dy,
                                              double              dAng                        );
    void  say                               ( const char * str                                );

    bool CheckLocalize();   //abc 13
    bool IsFallen();//abc 13
bool fallFront;
bool fallBack;
bool fallGhalt;
    //LOWLEVELACtIOn
    bool Rneck(bool start);
    bool Rneck_f(bool start);
    bool StandBack( bool start);
    bool StandFront( bool start);
    bool Ghalt( bool start);
    bool StandUp();
    bool walk(double,double,double );
    bool Sidewalk(SideT  );
    bool littlewalk(bool start);
    bool walkback(bool start);
    bool reset();
    bool turn_l(bool start) ;
    bool turn_r(bool start) ;
    bool move_r(bool start);
    bool move_l(bool start);
    bool kick_r(bool start);
    bool kick_l(bool start);

    //MIDLEVELACTION
    bool turnToRelativeAngle(AngDeg angTurn,double e);
    bool turnToAngle                        ( AngDeg              angTurn ,double e                    );
    bool turnToPoint                        ( VecPosition         posTurn  ,double e      ,bool start         );
	bool shoot();
	bool Check_kickingto(VecPosition goal);
	bool gotopos(VecPosition p,double e);
	bool gotopos_c(VecPosition p,double e);
	bool gotoposRelative(double e);
	bool goBehindeTheBall_forKick(double distance , double error , AngDeg error_angle);
	bool goBehindeTheBall_forKick_Relative(double distance  );
	bool SearchBall();
	void Say_Ball();
	void Say_Pass(ObjectT);
 bool kick(SideT side);
    bool Shoot(SideT side);
    bool TurntoAngle(AngDeg Angle);
    bool turn(SideT side);
    // Joint Utility methods
bool IsShootable                         (                                                );
bool IsPasable();
bool KickTo                             ();

bool GoToPos                            ( VecPosition pos,                  AngDeg angle  );
bool GoToBehindBall                     ( VecPosition ball      ,             AngDeg angle);
bool CloseToBall(double error);

	//JOINT
//	bool  adjustJoint                       ( JointT id,double   dSpeedMotor1, double dSpeedMotor2 = -1000.0   );
////    bool  neck                            ( AngDeg  valueFront, AngDeg  valueOpen, double  dSpeed = -1          );
//
//    bool  neck1                            ( AngDeg  valueFront, AngDeg  valueOpen, double  dSpeed  , double error);
//
//    bool  rotateLeg                         ( SideT side, AngDeg  value,
//                                                          double  dSpeed = -1                 );
//    bool  rotateArm                         ( SideT side, AngDeg  value,
//                                                          double  dSpeed = -1                 );
//    bool  knee                              ( SideT side, AngDeg  value,
//                                                          double  dSpeed = -1                 );
//    bool  elbow                             ( SideT side, AngDeg  value,
//                                                          double  dSpeed = -1                 );
//    bool  hip                               ( SideT side, AngDeg  valueFront,
//                                                          AngDeg  valueOpen,
//                                                          double  dSpeed = -1                 );
//    bool  shoulder                          ( SideT side, AngDeg  valueFront,
//                                                          AngDeg  valueOpen,
//                                                          double  dSpeed = -1                 );
//    bool  ankle                             ( SideT side, AngDeg  valueFront,
//                                                          AngDeg  valueOpen,
//                                                          double  dSpeed = -1                 );
//    bool  hip1                             ( SideT side, AngDeg  value,
//                                                          double  dSpeed = -1                 );


    // Basic Humanoid Abilities


	VecPosition Behind_ofPos_l(VecPosition pos , VecPosition goal);
	VecPosition Behind_ofPos_r(VecPosition pos , VecPosition Behind_of_Pos , AngDeg slope , double distance);
	VecPosition Behind_ofPos_u(VecPosition pos , AngDeg slope , double distance);
	VecPosition Behind_ofPos_d(VecPosition pos , AngDeg slope , double distance);
	bool Behind(VecPosition goal , double error);

    // Kicking Procedures
    bool kickHeuristic                      ( bool start                                    );

    // Learning Specific Methods
    bool initializeLearning                 (                                                 );
    void finalizeLearning                   (                                                 );

    void walkStraight                       (                                                 );

    ObjectT getNear_Number(VecPosition My);
    bool DoAct();
    bool Do_LAct();
    bool GotoPos(VecPosition target,double error,double MAX_Gain,double MIN_Gain);
    bool Inverse(VecPosition Hip, VecPosition Ankle, SideT side,double speed,double error=-1);
    bool Inverse_2(VecPosition  heap,VecPosition ankleLeft, VecPosition ankleRight,double speed,double error=-1);
    void ForwardFront(VecPosition );
    void ForwardOpen(VecPosition vec);
    bool step(double gain,double,double,SideT side);
    bool walkStop(double forwardGain,double sideGain,VecPosition);
    bool Sidestep(double gain,SideT side);
    bool	Head_Yaw						(double value,double gain,double error=-1);
     bool	Head_Pitch						(double value,double gain,double error=-1);
     bool 	LShoulder_Pitch					(double value,double gain,double error=-1);
     bool 	LShoulder_Roll					(double value,double gain,double error=-1);
     bool 	LElbow_Roll						(double value,double gain,double error=-1);
     bool 	LElbow_Pitch					(double value,double gain,double error=-1);
     bool 	RShoulder_Pitch					(double value,double gain,double error=-1);
     bool 	RShoulder_Roll					(double value,double gain,double error=-1);
     bool 	RElbow_Roll						(double value,double gain,double error=-1);
     bool 	RElbow_Pitch					(double value,double gain,double error=-1);
     bool 	LHip_YawPitch					(double value,double gain,double error=-1);
     bool 	LHip_Pitch						(double value,double gain,double error=-1);
     bool 	LHip_Roll						(double value,double gain,double error=-1);
     bool 	LKnee_Pitch						(double value,double gain,double error=-1);
     bool 	LAnkle_Picth					(double value,double gain,double error=-1);
     bool 	LAnkle_Roll						(double value,double gain,double error=-1);
     bool 	RHip_YawPitch					(double value,double gain,double error=-1);
     bool 	RHip_Pitch						(double value,double gain,double error=-1);
     bool 	RHip_Roll						(double value,double gain,double error=-1);
     bool 	RKnee_Pitch						(double value,double gain,double error=-1);
     bool 	RAnkle_Picth					(double value,double gain,double error=-1);
     bool 	RAnkle_Roll						(double value,double gain,double error=-1);
     bool GoToPoint(VecPosition point,double Error);
     bool Walk_Ball( );
     bool Front_Back_Walk(double Walk_gain);
     bool Front_Back_SWalk(double Walk_gain);
     bool Walk_Target(VecPosition goal);
     bool CheckForKick(VecPosition );
     bool Reset_Value();
     bool  neck1                            ( AngDeg  valueFront, AngDeg  valueOpen, double  dSpeed  , double error);

 	//JOINT
 	bool  adjustJoint                       ( JointT id,double   dSpeedMotor1, double dSpeedMotor2 = -1000.0   );

     bool  rotateLeg                         ( SideT side, AngDeg  value, double  dSpeed ,double error = -1 );
     bool  rotateArm                         ( SideT side, AngDeg  value, double  dSpeed ,double error = -1 );
     bool  knee                              ( SideT side, AngDeg  value, double  dSpeed ,double error = -1 );
     bool  elbow                             ( SideT side, AngDeg  value, double  dSpeed ,double error = -1 );
 //    bool  neck1                             ( 			  AngDeg  valueFront, AngDeg  valueOpen, double  dSpeed ,double error = -1  );
     bool  hip                               ( SideT side, AngDeg  valueFront, AngDeg  valueOpen, double  dSpeed ,double error = -1  );
     bool  shoulder                          ( SideT side, AngDeg  valueFront, AngDeg  valueOpen, double  dSpeed ,double error = -1  );
     bool  ankle                             ( SideT side, AngDeg  valueFront, AngDeg  valueOpen, double  dSpeed ,double error = -1  );
  public:
     SideT side_kick;
//     VecPosition Ball;
//     VecPosition Agent;
     AngDeg player_theta;
     VecPosition player,toop;
     bool isFirst , isFirst_kick , isFirst_shoot , isFirst_theta;
     double Speed;
     double delta;
     double Weight;
     double Lenght;
     bool Walk_Result;
 	 int KState;
 	 int Turn;
 	 AngDeg ang;
     int t1 ;
 	 int t2 ;
 	 bool b1;
 	 int TState ;
 	 bool b11 ;
 	 int GotoPos_State;
 	 bool result;
 	 int Behind_state;
 	int checkState;
	double gain;
//	static double gain=MIN_Gain;
	 int state ;//NECK
	 double walkSpeed;
	 double deltaSpeed;
	 AngDeg Angle;
    BasicAgent                              (                                                 );
  ~ BasicAgent                              (                                                 );
    Action this_action  ;	//abc 14
    Action last_action	;
    Lowlevel 	last_l_action ;
    Lowlevel l_action;
    VecPosition GP  ;
	double GP_gain;
	double rotate;
	double Gain;
//QUESTION static boodan ro chikar konam?
};

#endif // BASICAGENT_H
